/*= OX863 Metrix ========================================================= */
/* LabWindows Instrument Driver for OX863.                                 */
/* Original Release: November/12/96                                        */
/* By:  O. LEMAIRE                                                         */
/* Originally written in C                                                 */
/* Modification History:                                                   */
/*=========================================================================*/
#include "formatio.h"
#include "rs232.h"
#include "ox863.h"


/*= STATIC VARIABLES ======================================================*/
/* port contains the number of the port opened for the instrument module.  */
/* vert_mode contains the current vertical mode of the instrument.         */
/* ox863_err: the error variable for the instrument module                 */
/*=========================================================================*/
static short port;
static short vert_mode;
static short ox863_err;


/*= UTILITY ROUTINES ======================================================*/
int ox863_invalid_short_range (short val, short min, short max, int err_code);
int ox863_invalid_integer_range (int val, int min, int max, int err_code);
int ox863_invalid_longint_range (long val, long min, long max, int err_code);
int ox863_invalid_real_range (double val, double min, double max, int err_code);
int ox863_device_closed (void);
int ox863_send_command (int, int);

/*=========================================================================*/
/* This function opens a com port for the instrument module, queries for   */
/* ID, and initializes the instrument to a known state.                    */
/*=========================================================================*/
int ox863_init (int rs_port)
{
    char identifier[3];

    ox863_err = 0;
    if (ox863_invalid_integer_range (rs_port,1,8,-1) != 0){
        return ox863_err;
    }

    /*   Open And Configure RS232  */
    OpenComConfig (rs_port, "", 9600, 0, 8, 1, 512, 512);
    if (rs232err != 0) {
        ox863_err = rs232err;
        return ox863_err;
    }
    /*   Time-out : 1 Sec   */
    SetComTime (rs_port, 1.0);
    if (rs232err != 0) {
        ox863_err = rs232err+300;
        return ox863_err;
    }
    /*   Sets RTS/CTS ON   */
	SetCTSMode (rs_port, 1);
    if (rs232err != 0) {
        ox863_err = rs232err+300;
        return ox863_err;
    }
    port = rs_port;

    /*   Identify instrument   */
    if (ox863_send_command (OX863_ASK_ECHO, 0) !=0 ){
        ox863_close();
        return ox863_err;
    }
    ComRdTerm (port, identifier, 03, 04);
    if (identifier[1]!=0x31){
         ox863_close();
         return 223;
    }

    /*   Sets the Instrument with default values  */
    /*    Beam Finder   :   OFF             */
    /*    Vertical mode :   ALT             */
    /*    BWL           :   OFF				*/
    /*    CH1           : AC    0.1V/Div    */
    /*    Inversion CH2 :   OFF             */
    /*    CH2           : AC    0.1V/Div    */
    /*    Time base     : x10 OFF,TBA=50S/Div,DELAY OFF,B TRIG OFF,TBB=20S/DIV,1 STEP */
    /*    Trigger       : PTP ON,SLOPE +,AUTO ON,SOURCE CH1,COUPLING DC  */
    vert_mode = 2; /* ALT */
    if (ox863_display (2,0) != 0){
        return ox863_err;
    }
    if (ox863_channel (0, 0, 5, 0, 0) != 0){
        return ox863_err;
    }
    if (ox863_channel (1, 0, 5, 0, 0) !=0){
        return ox863_err;
    }
    if (ox863_timebase (0, 10, 0, 0, 11, 0) != 0){
        return ox863_err;
    }
    if (ox863_trigger (1, 0, 1, 0, 0) != 0){
        return ox863_err;
    }
    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                 Instrument Setup subroutine                         */
/*---------------------------------------------------------------------*/
int ox863_setup (int rw_config, char *file)
{
    int     file_id;
    short   i,tab_fonct[19];
    char    buff[20];

    tab_fonct[0] = OX863_WRI_CONF;
    tab_fonct[1] = OX863_VERTICAL_MODE;
    tab_fonct[2] = OX863_COUPLING_CH1;
    tab_fonct[3] = OX863_RANGE_CH2;
    tab_fonct[4] = OX863_COUPLING_CH2;
    tab_fonct[5] = OX863_INVERT_CH2;
    tab_fonct[6] = OX863_TB_DELAY;
    tab_fonct[7] = OX863_RANGE_TIME_BASE_A;
    tab_fonct[8] = OX863_RANGE_TIME_BASE_B;
    tab_fonct[9] = OX863_TRIGGER_B;
    tab_fonct[10] = OX863_EXP10;
    tab_fonct[11] = OX863_TRIGGER_AUTO;
    tab_fonct[12] = OX863_TRACE_SEP;
    tab_fonct[13] = OX863_TRIGGER_COUPLING;
    tab_fonct[14] = OX863_TRIGGER_SOURCE;
    tab_fonct[15] = OX863_TRIGGER_PTP;
    tab_fonct[16] = OX863_TRIGGER_SLOPE;
    tab_fonct[17] = OX863_BWL;
    tab_fonct[18] = OX863_RANGE_CH1;

    ox863_err = 0;
    if (ox863_device_closed() != 0 ){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (rw_config,0,1,-1) != 0){
        return ox863_err;
    }
    if (!rw_config)  {
        /* Save */
        file_id = OpenFile (file, 2, 0, 1);
        if (file_id ==  -1)  {
            ox863_err =  -2;
            return ox863_err;
        }
        /* Input Buffer Empty */
        FlushInQ (port);
        if (rs232err != 0) {
            ox863_err = rs232err+300;
            return ox863_err;
        }
        if (ox863_send_command (OX863_READ_SETUP, 0) != 0 ){
            return ox863_err;
        }
        i = 0;
        while (i < 19)  {
            ComRdByte (port);
            if (rs232err != 0)  {
                ox863_err = rs232err+300;
                return ox863_err;
            }
            buff[i] = ComRdByte (port);
            if (rs232err != 0)  {
                ox863_err = rs232err+300;
                return ox863_err;
            }
            ComRdByte (port);
            if (rs232err != 0)  {
                ox863_err = rs232err+300;
                return ox863_err;
            }
            i++;
        }
        /* Invert BWL and DELAY */
        buff[19] = buff[17];
        buff[17] = buff[6];
        buff[6] = buff[19];
        buff[19] = 0;
        /* Invert VERTICAL MODE and RANGE CH1 */
        buff[19] = buff[18];
       	buff[18] = buff[1];
       	buff[1] = buff[19];
       	buff[19] = 0;

        if (WriteFile (file_id, buff, 19) == -1){
            ox863_err =  230;
            return ox863_err;
        }
        if (CloseFile (file_id) == -1){
            ox863_err = -2 ;
            return ox863_err;
        }
    }
    else  {
        /* Recall */
        file_id = OpenFile (file, 1, 0, 1);
        if (file_id ==  -1)  {
            ox863_err =  -2;
            return ox863_err;
        }
        if (ReadFile (file_id, buff, 19) ==  -1)  {
            ox863_err =  231;
            return ox863_err;
        }
        i = 0;
        vert_mode = buff[1] - 0x30;
        while (i < 19)  {
            if (ox863_send_command (tab_fonct[i], buff[i] - 0x30) != 0 ){
                return ox863_err;
            }
            i++;
        }
        if (CloseFile (file_id) == -1 ){
            ox863_err =  -2;
            return ox863_err;
        }

    }
    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                 Instrument ReadOut Setup subroutine                 */
/*---------------------------------------------------------------------*/
int ox863_readout (char *file)
{
    int     file_id;
    short   i,tab_fonct[13];
    char    buff[20];

    tab_fonct[0] = OX863_REMOTE;	 
    tab_fonct[1] = OX863_READ_OUT;
    tab_fonct[2] = OX863_PROBE_CH1;
    tab_fonct[3] = OX863_PROBE_CH2; 	
    tab_fonct[4] = OX863_MEASUREMENT;
    tab_fonct[5] = OX863_CURSOR1;		
    tab_fonct[6] = OX863_CURSOR2;		
    tab_fonct[7] = OX863_CURSOR3;		
    tab_fonct[8] = OX863_STANDARD_LO;
    tab_fonct[9] = OX863_STANDARD_HI;
    tab_fonct[10] = OX863_LINE_LO;
    tab_fonct[11] = OX863_LINE_HI;		

    ox863_err = 0;
    if (ox863_device_closed() != 0 ){
        return ox863_err;
    }

    /* Save */
    file_id = OpenFile (file, 2, 0, 1);
    if (file_id ==  -1)  {
        ox863_err =  -2;
        return ox863_err;
    }
    /* Input Buffer Empty */
    FlushInQ (port);
    if (rs232err != 0) {
        ox863_err = rs232err+300;
        return ox863_err;
    }
    if (ox863_send_command (OX863_READ_READOUT, 0) != 0 ){
        return ox863_err;
    }
    i = 0;
    while (i < 12)  {
        ComRdByte (port);
        if (rs232err != 0)  {
            ox863_err = rs232err+300;
            return ox863_err;
        }
        buff[i] = ComRdByte (port);
        if (rs232err != 0)  {
            ox863_err = rs232err+300;
            return ox863_err;
        }
        ComRdByte (port);
        if (rs232err != 0)  {
            ox863_err = rs232err+300;
            return ox863_err;
        }
        i++;
    }

    if (WriteFile (file_id, buff, 12) == -1){
        ox863_err =  230;
        return ox863_err;
    }
    if (CloseFile (file_id) == -1){
        ox863_err = -2 ;
        return ox863_err;
    }
    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                  Autoset subroutine                                 */
/*---------------------------------------------------------------------*/
int ox863_autoset (void)
{
    ox863_err = 0;
    if (ox863_device_closed() != 0 ){
        return ox863_err ;
    }

    if (ox863_send_command(OX863_COM_AUTOSET,0) !=0 ){
        return ox863_err;
    }
    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                  Display settings subroutine                        */
/*---------------------------------------------------------------------*/
int ox863_display (int mod,int bwl)
{
    ox863_err = 0;
    if (ox863_device_closed() != 0 ){
        return ox863_err ;
    }
    if (ox863_invalid_integer_range (mod,0,5,-1) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (bwl,0,1,-2) != 0){
        return ox863_err;
    }
    if (ox863_send_command (OX863_VERTICAL_MODE, mod) != 0){
        return ox863_err;
    }
    vert_mode = mod;
    /* Bwl */
    if (ox863_send_command (OX863_BWL, bwl) != 0){
        return ox863_err;
    }
    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                 Channel Settings subroutine                         */
/*---------------------------------------------------------------------*/
int ox863_channel (int channel_num, int coupling, int sensibility,
					int inversion, int probe)
{
    ox863_err = 0;
    if (ox863_device_closed() != 0 ){
        return ox863_err ;
    }
    if (ox863_invalid_integer_range (channel_num,0,1,-1) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (coupling,0,2,-2) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (sensibility,0,10,-3) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (inversion,0,1,-4) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (probe,0,2,-5) != 0){
        return ox863_err;
    }
    if (!channel_num){
        if (ox863_send_command (OX863_COUPLING_CH1, coupling) != 0)
            return ox863_err;
    }
    else{
        if (ox863_send_command (OX863_COUPLING_CH2, coupling) != 0)
            return ox863_err;
    }
    if (!channel_num){
        if (ox863_send_command (OX863_RANGE_CH1, sensibility) != 0)
            return ox863_err;
    }
    else{
        if (ox863_send_command (OX863_RANGE_CH2, sensibility) != 0){
            return ox863_err;
        }
    }
    if (!channel_num){
        if (ox863_send_command (OX863_PROBE_CH1, probe) != 0)
            return ox863_err;
    }
    else{
        if (ox863_send_command (OX863_PROBE_CH2, probe) != 0){
            return ox863_err;
        }
    }
    /* Channel invert */
    if (ox863_send_command (OX863_INVERT_CH2, inversion) !=0){
        return ox863_err;
    }

    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                     Time base subroutine                            */
/*---------------------------------------------------------------------*/
int ox863_timebase (int expend,int range_A,int dlay,int B_trig,int range_B,int nb_step)
{

    ox863_err = 0;
    if (ox863_device_closed() != 0 ){
        return ox863_err ;
    }
    if (ox863_invalid_integer_range (expend,0,1,-1) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (range_A,0,19,-2) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (dlay,0,2,-3) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (B_trig,0,1,-4) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (range_B,0,19,-5) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (nb_step,0,15,-6) != 0){
        return ox863_err;
    }
    /* if XY mode, error -7 */
    if (vert_mode == 5)  {
        ox863_err =  -7;
        return ox863_err ;
    }
    if (ox863_send_command (OX863_EXP10, expend) != 0){
        return ox863_err;
    }
    if (ox863_send_command (OX863_RANGE_TIME_BASE_A, range_A) != 0){
        return ox863_err;
    }
    if (ox863_send_command (OX863_TB_DELAY, dlay) != 0){
        return ox863_err;
    }
    /* if mode delay or alterned */
    if (dlay != 0) {
        /* if Range B <= Range A, error -8 */
        if (range_B < range_A) {
            ox863_err = -8;
            return ox863_err;
        }
        if (ox863_send_command (OX863_RANGE_TIME_BASE_B, range_B) != 0){
            return ox863_err;
        }
        if (ox863_send_command (OX863_TRIGGER_B, B_trig) != 0){
             return ox863_err;
        }
        if (ox863_send_command (OX863_TRACE_SEP, nb_step) != 0){
             return ox863_err;
        }
    }
    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                     Trigger subroutine                              */
/*---------------------------------------------------------------------*/
int ox863_trigger (int ptp,int slope,int mod_auto,int source,int coupling)
{
    ox863_err = 0;
    if (ox863_device_closed() != 0 ){
        return ox863_err ;
    }
    if (ox863_invalid_integer_range (ptp,0,1,-1) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (slope,0,1,-2) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (mod_auto,0,1,-3) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (source,0,4,-4) != 0){
        return ox863_err;
    }
    if (ox863_invalid_integer_range (coupling,0,5,-5) != 0){
        return ox863_err;
    }
    /* if XY mode, error= -6 */
    if (vert_mode == 5)  {
        ox863_err =  -6;
        return ox863_err ;
    }
    if (ox863_send_command (OX863_TRIGGER_SOURCE, source) != 0){
        return ox863_err;
    }
    if (source != 3){
        if (ox863_send_command (OX863_TRIGGER_COUPLING, coupling) != 0){
            return ox863_err;
        }
    }
    /* if SOURCE LINE, COUPLING OFF */
    else {
        if (ox863_send_command (OX863_TRIGGER_COUPLING, 6) != 0){
            return ox863_err;
        }
    }
    if (ox863_send_command (OX863_TRIGGER_PTP, ptp) != 0){
        return ox863_err;
    }
    if (ox863_send_command (OX863_TRIGGER_AUTO, mod_auto) != 0){
        return ox863_err;
    }
    if (ox863_send_command (OX863_TRIGGER_SLOPE, slope) != 0){
        return ox863_err;
    }
    return ox863_err;
}
/*---------------------------------------------------------------------*/
/*                     Measure subroutine                              */
/*---------------------------------------------------------------------*/
int  ox863_measure (int measure, int cursor1, int cursor2, int cursor3)
	{
    /*  Check for device closed */
    if (ox863_device_closed() !=0)
        return ox863_err;

    if (ox863_invalid_integer_range (measure,0,3,-1) != 0)
        return ox863_err;
    if (measure == 1)
		{
		if (ox863_invalid_integer_range (cursor1,0,150,-2) != 0)
        	return ox863_err;
    	if (ox863_invalid_integer_range (cursor2,0,150,-3) != 0)
        	return ox863_err;
        }
    else if(measure>1)
    	{
    	if (ox863_invalid_integer_range (cursor1,0,230,-2) != 0)
        	return ox863_err;
	    if (ox863_invalid_integer_range (cursor2,0,230,-3) != 0)
	        return ox863_err;
	    if (ox863_invalid_integer_range (cursor3,0,230,-4) != 0)
	        return ox863_err;
		}
		
    if (ox863_send_command (OX863_MEASUREMENT, measure) != 0)
        return ox863_err;
    
    if(measure==1)
	    {
	    if (ox863_send_command (OX863_CURSOR1, cursor1) != 0)
	        return ox863_err;
	    if (ox863_send_command (OX863_CURSOR2, cursor2) != 0)
	        return ox863_err;
	    }
	else if(measure>1)
	    {
	    if (ox863_send_command (OX863_CURSOR1, cursor1 - 23) != 0)
	        return ox863_err;
	    if (ox863_send_command (OX863_CURSOR2, cursor2 - 23) != 0)
	        return ox863_err;
    	if(measure==3)
		    if (ox863_send_command (OX863_CURSOR3, cursor3 - 23) != 0)
		        return ox863_err;
    	}
    	
    return ox863_err;
    }
/*---------------------------------------------------------------------*/
/*                     Standard TV subroutine                          */
/*---------------------------------------------------------------------*/
int  ox863_standard (int standard)
	{
    /*  Check for device closed */
    if (ox863_device_closed() !=0)
        return ox863_err;
    
    if (ox863_invalid_integer_range (standard,1,1250,-1) != 0)
        return ox863_err;

    if (standard%100 == 0)
    	{
	    if (ox863_send_command (OX863_STANDARD_HI, standard/100) != 0)
	        return ox863_err;
	    if (ox863_send_command (OX863_STANDARD_LO, standard%100) != 0)
    	    return ox863_err;
    	}
    else
    	{
	    if (ox863_send_command (OX863_STANDARD_LO, standard%100) != 0)
    	    return ox863_err;
    	if (ox863_send_command (OX863_STANDARD_HI, standard/100) != 0)
        	return ox863_err;
        }
    
    return ox863_err;
    }
/*---------------------------------------------------------------------*/
/*                     Line TV subroutine                              */
/*---------------------------------------------------------------------*/
int  ox863_line (int line)
	{
    /*  Check for device closed */
    if (ox863_device_closed() !=0)
        return ox863_err;
    
    if (ox863_invalid_integer_range (line,1,1250,-1) != 0)
        return ox863_err;

    if (line%100 == 0)
		{   
	    if (ox863_send_command (OX863_LINE_HI, line/100) != 0)
	        return ox863_err;
	    if (ox863_send_command (OX863_LINE_LO, line%100) != 0)
	        return ox863_err;
		}
	else
		{
	    if (ox863_send_command (OX863_LINE_LO, line%100) != 0)
	        return ox863_err;
	    if (ox863_send_command (OX863_LINE_HI, line/100) != 0)
	        return ox863_err;
		}
    return ox863_err;
    }
/*----------------------------------------------------------------------*/
/* This function closes the port for the instrument module and sets the */
/* port to zero.                                                        */
/*----------------------------------------------------------------------*/
int ox863_close (void)
{
    /*  Check for device closed */
    if (ox863_device_closed() !=0){
        return ox863_err;
    }

    /* Wait for Empty output buffer */
    while ( GetOutQLen(port) != 0) {};
    /* Close the com port.  If error, set ox863_err = rs232err+300.*/
    CloseCom(port);
    if (rs232err != 0) {
        ox863_err = rs232err+300;
        return ox863_err;
    }
    port = 0;
    return ox863_err;
}
/* = UTILITY ROUTINES =====================================================*/
/*=========================================================================*/
/* Function: Invalid Short Range                                           */
/* Purpose:  This function checks a short to see if it lies between a      */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox863_invalid_short_range (short val, short min, short max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox863_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox863_invalid_integer_range (int val, int min, int max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox863_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int ox863_invalid_longint_range (long val, long min, long max, int err_code)
{
    if (val < min || val > max) {
        ox863_err = err_code;
        return -1;
    }
    return 0;
}
/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox863_invalid_real_range (double val, double min, double max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox863_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, a 1 is       */
/*           returned, 0 otherwise.                                        */
/*=========================================================================*/
int ox863_device_closed (void)
{
    if (port== 0){
        ox863_err = 232;
        return -1;
    }
    return 0 ;

}
/*=========================================================================*/
/* Function: Send Command                                                  */
/* Purpose: This function sends a command to the instrument. If the        */
/*          scope is not initialized, an error 232 is returned.            */
/*=========================================================================*/
int ox863_send_command (int function, int parameter)
{
    ox863_err = 0;
    if (port != 0)  {
        parameter += 0x30;
        if (ComWrtByte (port, function) ==0){
            ox863_err = 230;
            return ox863_err ;
        }
        if (ComWrtByte (port, parameter) == 0){
            ox863_err = 230;
            return ox863_err ;
        }
        if (ComWrtByte (port, 04) ==0){
            ox863_err = 230;
            return ox863_err ;
        }
    }
    else{
        ox863_err = 232;
    }
    return ox863_err ;
}
/*= THE END ===============================================================*/
